# frozen_string_literal: true

require_relative 'base'
require_relative 'templates'

module Overture
  module Generators
    class App < Base
      def call(vars = {})
        @app_dir = vars[:app_name].gsub(' ', '_').downcase

        create_files(vars)
        run_setup
      end

      private

      attr_reader :app_dir

      def create_files(vars)
        bin(vars)
        config(vars)
        lib(vars)
        config_ru
        gemfile
        procfile
        rakefile
      end

      def run_setup
        shell "cd #{app_dir} && bundle install"
      end

      def bin(vars)
        bin_vars = vars.fetch(:bin) { {} }

        bin_console(bin_vars)
        bin_dev_http_api(bin_vars)
      end

      def config(vars)
        config_vars = vars.fetch(:config) { {} }

        config_environment(config_vars)
        config_puma(config_vars)
      end

      def lib(vars)
        lib_vars = vars.fetch(:lib) { {} }

        mkdir("#{app_dir}/lib/interactors")
        mkdir("#{app_dir}/lib/models")

        lib_http_api_server(lib_vars)
        lib_http_api_endpoints(lib_vars)
      end

      def config_ru(vars = {})
        content = compile(Templates[:config_ru], vars)
        write(content, "#{app_dir}/config.ru")
      end

      def gemfile(vars = {})
        content = compile(Templates[:gemfile], vars)
        write(content, "#{app_dir}/Gemfile")
      end

      def procfile(vars = {})
        content = compile(Templates[:procfile], vars)
        write(content, "#{app_dir}/Procfile")
      end

      def rakefile(vars = {})
        content = compile(Templates[:rakefile], vars)
        write(content, "#{app_dir}/Rakefile")
      end

      def bin_console(vars = {})
        content = compile(Templates[:bin_console], vars)

        path = write(content, "#{app_dir}/bin/console")
        chmod('+x', path)
      end

      def bin_dev_http_api(vars = {})
        content = compile(Templates[:bin_dev_http_api], vars)

        path = write(content, "#{app_dir}/bin/dev_http_api")
        chmod('+x', path)
      end

      def config_environment(vars = {})
        content = compile(Templates[:config_environment], vars)
        write(content, "#{app_dir}/config/environment.rb")
      end

      def config_puma(vars = {})
        content = compile(Templates[:config_puma], vars)
        write(content, "#{app_dir}/config/puma.rb")
      end

      def lib_http_api_server(vars = {})
        content = compile(Templates[:lib_http_api_server], vars)
        write(content, "#{app_dir}/lib/http_api/server.rb")
      end

      def lib_http_api_endpoints(vars = {})
        content = compile(Templates[:lib_http_api_endpoints], vars)
        write(content, "#{app_dir}/lib/http_api/endpoints.rb")
      end
    end
  end
end
