# frozen_string_literal: true

require 'erb'
require 'fileutils'

module Overture
  module Generators
    class Base
      def initialize(config = {})
        @config = config
      end

      def compile(template, vars = {})
        ERB.new(template).result_with_hash(vars)
      end

      # TODO: check existence and ask for overwrite
      def write(content, filepath)
        location = File.expand_path(filepath)
        dirname = File.dirname(location)

        mkdir(dirname)

        File.open(location, 'w') do |f|
          f.puts content
        end

        puts "  created file: #{filepath}"
        filepath
      end

      def mkdir(path)
        FileUtils.mkdir_p(path) unless File.directory?(path)
      end

      def touch(filepath)
        write('', filepath)
      end

      def chmod(mod, location)
        FileUtils.chmod(mod, File.expand_path(location))
      end

      def shell(command)
        command = command.join(' && ') if command.is_a?(Array)
        puts "  running: #{command}"
        `#{command}`
      end

      protected

      attr_reader :config
    end
  end
end
