# frozen_string_literal: true

module Overture
  module HttpApi
    module Helpers
      def json(data, serializer: nil, options: {})
        data = data.all if data.respond_to?(:all)
        return JSON.generate(data, options) unless serializer

        serializer.new(data, options).serialized_json
      end

      def halt_not_found!
        halt_error!(404, 'Not Found')
      end

      def halt_unauthorized!
        halt_error!(401, 'Unauthorized')
      end

      def halt_not_acceptable!
        halt_error!(406, 'Not Acceptable')
      end

      def halt_unsupported_media_type!
        halt_error!(415, 'Unsupported Media Type')
      end

      def halt_unprocessable!(error)
        halt_error!(422, error)
      end

      def halt_server_error!(error = 'Server Error')
        halt_error!(500, error)
      end

      def halt_error!(status, title)
        halt status, json(
          errors: [{
            status: status.to_s,
            title: title
          }]
        )
      end

      def require_params!(*names)
        names.each do |name|
          halt_unprocessable!("#{name} is required") unless params.key?(name)
        end
      end

      def found!(object)
        object.nil? ? halt_not_found! : object
      end
    end
  end
end
