# frozen_string_literal: true

require 'sinatra/base'
require 'rack/parser'
require_relative 'helpers'

module Overture
  module HttpApi
    class Server < Sinatra::Base
      helpers Helpers

      MIME_TYPE = 'application/vnd.api+json'

      def notify_error(error)
        puts "[ERROR] #{error.inspect}"
      end

      configure do
        enable :logging
        enable :protection
        disable :raise_errors
        set :dump_errors, settings.development?
        disable :show_exceptions

        use Rack::Parser
      end

      before do
        content_type MIME_TYPE

        non_charset = request.media_type_params.keys.any? { |k| k != 'charset' }
        halt_unsupported_media_type! if request.media_type != MIME_TYPE || non_charset
        halt_not_acceptable! unless request.preferred_type.entry == MIME_TYPE || request.options?
      end

      not_found do
        halt_not_found!
      end

      error do
        exception = env['sinatra.error']

        notify_error(exception)

        halt_server_error!(exception.message)
      end
    end
  end
end
