# frozen_string_literal: true

require 'sequel'

Sequel.extension :migration

namespace :db do
  desc 'Run migrations for $DATABASE_URL'
  task :migrate, [:version] do |_t, args|
    require_relative '../overture/database'

    DB ||= Overture::Database.connect(ENV['DATABASE_URL'])
    version = args[:version].to_i if args[:version]

    puts "Migrating database to #{version ? "version #{version}" : 'latest version'}..."
    Sequel::Migrator.run(DB, 'migrations', target: version)
  end

  desc 'Resets $DATABASE_URL schema to latest version'
  task :reset do
    Rake::Task['db:migrate'].reenable
    Rake::Task['db:migrate'].invoke(0)
    Rake::Task['db:migrate'].reenable
    Rake::Task['db:migrate'].invoke
  end

  namespace :test do
    desc 'Resets $TEST_DATABASE_URL database schema to latest version'
    task :reset do
      ENV['RACK_ENV'] = 'test'
      ENV['DATABASE_URL'] = ENV['TEST_DATABASE_URL']

      Rake::Task['db:migrate'].reenable
      Rake::Task['db:migrate'].invoke(0)
      Rake::Task['db:migrate'].reenable
      Rake::Task['db:migrate'].invoke
    end
  end
end
